######################################################################
##### Code for Day 2 of SEM Workshop
##### covering ML fitting of SEMs of observed variables
#####
##### Jarrett E.K. Byrnes
#####
##### Last Modified 2/1/13
######################################################################

#Set your working Directory
#setwd("~/Dropbox/SEM/lavaan")
library(lavaan)

##############################################################################################################################
# Multigroup Analysis
##############################################################################################################################

meadows<-read.csv("./FinnishMeadows_Multigroup.csv")

meadowModel<-'rich ~ elev + mass
              mass ~ me*elev'

#different fits - one whole shebang, once grouped with everything varying, one grouped, but with everything constrained
meadowFit<-sem(meadowModel, data=meadows)

coef(meadowFit)
summary(meadowFit)

#OK, now introduce groups
meadowFitFree<-sem(meadowModel, data=meadows, group="grazed")
coef(meadowFitFree)

summary(meadowFitFree)

#introducing constraints
meadowFitEqual<-sem(meadowModel, data=meadows, group="grazed", group.equal=c( "intercepts", "means", "regressions", "residuals", "residual.covariances"))

#is the constraint OK?
anova(meadowFitFree, meadowFitEqual)

#constrain just the elev-mass relationship
meadowModel2<-'rich ~ elev + mass
              mass ~ c("me", "me")*elev'

meadowFitFree2<-sem(meadowModel2, data=meadows, group="grazed")

coef(meadowFitFree2)
summary(meadowFitFree2)

#test to see if this is one of the culprits!
anova(meadowFitFree, meadowFitFree2)

###############
#Exercise- split by par2
###############
meadowFitFreePar2<-sem(meadowModel, data=meadows, group="par2")
meadowFitEqualPar2<-sem(meadowModel, data=meadows, group="par2", group.equal=c( "intercepts", "means", "regressions", "residuals", "residual.covariances"))

summary(meadowFitFreePar2)
anova(meadowFitEqualPar2, meadowFitFreePar2)

#see if its the same constraint
meadowFitFreePar2_2<-sem(meadowModel2, data=meadows, group="par2")
anova(meadowFitFreePar2, meadowFitFreePar2_2)

#is that the only contraint?
anova(meadowFitEqualPar2, meadowFitFreePar2_2)

coef(meadowFitFreePar2_2)

######
# Exercise: Try other constraints & group - here's one
######

meadowModel3<-'rich ~ c("re", "re")*elev + mass
              mass ~ elev'

meadowFitFreePar2_3<-sem(meadowModel3, data=meadows, group="par2")

#does it matter?  Yes!
anova(meadowFitFreePar2, meadowFitFreePar2_3)

#see others that may vary
coef(meadowFitFreePar2_3)

######
# Exercise: Try other constraints & group - here's one
######


meadowModelNoLabel<-'rich ~ elev + mass
              mass ~ elev'


meadowFitOneFree<-sem(meadowModelNoLabel, data=meadows, group="grazed", 
                      group.equal="regressions",
                      group.partial="mass ~ elev")


summary(meadowFitOneFree)

meadowFitTwoFree<-sem(meadowModelNoLabel, data=meadows, group="grazed", 
        group.equal="regressions",
        group.partial=c("mass ~ elev", "rich ~ mass"))

inspect(meadowFitTwoFree, "fit")

#compare to constrained model
anova(meadowFitTwoFree, meadowFitFree)


##############################################################################################################################
# Latent Variables
##############################################################################################################################

####
#Multiple Indicators - Santos Example
####

santosCov <-read.table("./santosCov.txt", na.strings=".")
santosCov <-as.matrix(santosCov)

santosCFA1<-'
  Aposematism =~Alkaloid.quantity+Alkaloid.diversity+Conspicuous.coloration
'

santosFit1<-sem(santosCFA1, sample.cov=santosCov, sample.nobs=21)
summary(santosFit1)
standardizedSolution(santosFit1)	

####
# Exercise - body size
####

santosSize<-'
  Size =~ Log.Mass + Log.RMR + Log.Scope'

santosSizeFit<-sem(santosSize, sample.cov=santosCov, sample.nobs=21)

summary(santosSizeFit, standardized=T)

####
#Two variable CFA
####

santosCFA2<-paste(santosCFA1, '
		Aposematism =~ Ant.Mite.Specialization+log.Prey
                  Scale =~Log.Mass+Log.RMR+Log.Scope+Conspicuous.coloration
                  ', sep="\n")
	

santosFit<-sem(santosCFA2, sample.cov=santosCov, sample.nobs=21)
summary(santosFit)
standardizedSolution(santosFit)


#####################################################
### Spartina SEM
#############################################

spartina <- read.csv("./TravisDataForLVExample.csv")


#what if an exogenous variable affects indicators of a latent variable
spartinaModel<-'performance =~  clonediam + numbstems + numbinfl + meanht + meanwidth
               meanht ~~ meanwidth

               performance ~ geneticdist
'

spartinaFit<-sem(spartinaModel, data=spartina)

summary(spartinaFit, standardized=T, rsquare=T)

############################
## Spartina Exercise
############################

spartinaModel2<-paste(spartinaModel, '
                      meanht ~ latitude
                      meanwidth ~ latitude', sep="\n")

spartinaFit2<-sem(spartinaModel2, data=spartina)

summary(spartinaFit2, standardized=T, rsquare=T)

############################
# Kelp Measurement Error
############################

lter<-read.csv("./lter_kelp.csv")

#1) Calculate fitted values for spring biomass
#landsat observations to biomass
lter$landsat_spring_biomass<-154.89*lter$spring_canopy+68.62

#2) Calculate fitted values for summer biomass
#summer kelp counts to biomass y=0.08x+0.01 r^2=0.79
lter$summer_kelp_biomass<-0.08*lter$kelp+0.01


#3) Transform fitted values for easier fitting
#transformation for easier fitting
lter$summer_kelp_biomass<-log(lter$summer_kelp_biomass +1)
lter$landsat_spring_biomass <-log(lter$landsat_spring_biomass +1)

#No measurement error
noerror<-'summer_kelp_biomass ~ landsat_spring_biomass'
noerrorFit<-sem(noerror, data=lter)
summary(noerrorFit, standardized=T)

#measurement error in predictor
var(lter$landsat_spring_biomass, na.rm=T)*(0.38)

#[1] 3.762301

errorCanopy<-'
  true_spring_biomass =~ 1*landsat_spring_biomass

  summer_kelp_biomass ~ true_spring_biomass

#error
landsat_spring_biomass ~~ 3.762301*landsat_spring_biomass
'

errorCanopyFit<-sem(errorCanopy, data=lter)
summary(errorCanopyFit, standardized=T)

##error in both
var(lter$summer_kelp_biomass, na.rm=T)*(0.21)
#[1] 0.5495345

errorBoth<-'
  true_spring_biomass =~ 1*landsat_spring_biomass
	true_summer_biomass =~ 1*summer_kelp_biomass
	
	true_summer_biomass ~ true_spring_biomass
	
	#error
	landsat_spring_biomass ~~ 3.762301*landsat_spring_biomass
	summer_kelp_biomass ~~ 0.5495345* summer_kelp_biomass
'

errorBothFit<-sem(errorBoth, data=lter)
summary(errorBothFit, standardized=T)